;----------------------------------------------------------------------------------------;
;               SIMULATION OF PEDESTRIAN TRAFFIC IN COUNTERFLOW                          ;
;      This agent-based model simulates a pedestrian crowd in a corridor                 ;
;                                   March 2011                                           ;
; A.L Ballinas-Hernandez, anage3@hotmail.com                                             ;
; A. Munoz-Melendez, munoz@inaoep.mx                                                     ;
; A. Rangel-Huerta, arangel@cs.buap.mx                                                   ;
;                                                                                        ;
; Authors give permission for the use of this code for scholarly purposes.               ;
; They would apreciate a notification of modification of any part of this code.          ;
;                                                                                        ;
; A detailed description of this work is provided in:                                     ;
;                                                                                        ;
; Ballinas-Hernnez, A, Muoz-Melndez, A, Rangel-Huerta, A (2011). Multiagent System    ;
; Applied to the Modeling and Simulation of Pedestrian Traffic in Counterflow. Journal   ;
; of Artificial Societies and Social Simulation 14 (3) 2.                                ;
;----------------------------------------------------------------------------------------;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;               E N V I R O N M E N T    C O N F I G U R A T I O N                     ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

globals [ x-aux y-aux wall step-size-sum act-sum average-speed-iter total-average-speed average-act-iter 
total-average-act pref-turn-free-path pref-turn-obs-path pref-turn-pedes-encs pref-turn-combin-encs setup?
max-pedestrians increase-pedestrians] 

;;; Setup procedures ;;;
to setup
  clear-all
  print "MOBS program: Multi-agent Organized pedestrians with Built-in Specialization functions"
  clear-output 
  ask turtles [ die ]
  ask patches [ set pcolor 7 ]
  draw-walls     
  set increase-pedestrians 10
  set number-pedestrians increase-pedestrians
  set average-act-iter 0
  set x-aux 0
  set setup? true
  set max-pedestrians (corridor-width * 100)
  print (word "Maximum number of pedestrians can walk on the corridor is: " max-pedestrians)
end

;;; Draw walls according to the width corridor ;;;
to draw-walls
  ; top wall
  set wall -51 
  repeat 101 [ ask patch wall((corridor-width + 1) / 2) [ set pcolor black ] set wall( wall + 1) ]
  
  ; down wall
  set wall -51 
  repeat 101 [ ask patch wall(-((corridor-width + 1) / 2)) [ set pcolor black ] set wall( wall + 1) ]
end

;;; Restart run procedures ;;;
to restart
  ask turtles [ die ]
  allocate-pedestrians (number-pedestrians)
  set average-speed-iter 0   
  set average-act-iter 0   
end

;;; Run time procedures ;;;
to go
  set step-size-sum 0
  set act-sum 0 
  ask turtles [ walk ] 
  set average-speed-iter (average-speed-iter + (step-size-sum / count turtles))
  set average-act-iter   (average-act-iter + (act-sum / count turtles))
   
  tick
  ;; When an execution is completed, restart increasing the density of occupation
  if (ticks mod max-iterations) = 0 [
     
     ;; Calculate average speed for each execution
     set total-average-speed (average-speed-iter / max-iterations) * 0.7
     set-current-plot "Average-speed-vs-density"
     plotxy number-pedestrians total-average-speed
     
     ;; Calculate average activity for each execution
     set total-average-act (average-act-iter / max-iterations)
     set-current-plot "Average-activity-vs-density"
     plotxy number-pedestrians total-average-act
     
     let n-pedestrians (number-pedestrians + increase-pedestrians)
     
     ifelse (n-pedestrians <= max-pedestrians) [
           set number-pedestrians (number-pedestrians + increase-pedestrians)
           print "MOBS program: Multi-agent Organized pedestrians with Built-in Specialization functions"
           print (word "Total number of pedestrians: " number-pedestrians)
           restart
           
        ]
        [show "----------------------------------------------------------"
         show "MOBS execution ends with the maximum number of pedestrians"
         stop
     ]
  ]
end

;;; Create and distribute pedestrians along to the corridor ;;;
to add-pedestrians
     print "MOBS program: Multi-agent Organized pedestrians with Built-in Specialization functions"
     allocate-pedestrians (number-pedestrians)
     preference-factors-turn
     reset-timer
     set number-pedestrians count turtles
     print (word "Total number of pedestrians: " count turtles)
end

to allocate-pedestrians [n]
  repeat n [  
     ifelse count turtles + 1 <= max-pedestrians
        [create-pedestrian]
        [ show "----- Maximum number of pedestrians on the corridor is reached -----"
          stop
        ]
  ]
end

;;; Create pedestrians in different directions (walking left or right) ;;;
to create-pedestrian
  let throw-coin (random 2)
  let aux-y-goal (random ((corridor-width + 1) - 2)) - (((corridor-width + 1) - 2) / 2)
  
  ;; Throw-coin is a variable used to create a pedestrian oriented with any direction
  ifelse (throw-coin = 0) 
  [ 
   find-location 100 (corridor-width + 1) -1
   ;; Create a pedestrian walking left 
   crt 1
     [ set color red
       set size 1     
       setxy x-aux y-aux 
       set heading 90
       
       ;; Define the agent shape according to its walking preferences
       ifelse pedestrians-profile = "Relaxed"     ;; "Relaxed" pedestrians do not want to reach their goal quickly
       [ 
         set shape "relaxed"
       ]
       [ ifelse pedestrians-profile = "Standard" ;; "Standard" pedestrians perform normal walking for different situations
          [ 
            set shape "peaton"
          ]
          [ if pedestrians-profile = "Hurried"  ;; "Relaxed" pedestrians do not want to reach their goal quickly
              [ 
                set shape "hurried"
              ]
          ]
       ]
       
       set speed (random 2) 
       set x-goal 50
       set y-goal aux-y-goal
       set front-partners 0
       set lateral-partners 0
       set perceptual-field-i 0
       set front-perceptual-field "empty"
       set count-free-path 0
       set count-obs-path 0
       set count-pedestrian-enc 0
       set count-combined-enc 0
       set count-overlap 0
     ]
  ] 
  [
   find-location 100 (corridor-width + 1) 1
   ;; Create a pedestrian walking right
   crt 1
     [ set color blue
       set size 1     
       setxy x-aux y-aux 
       set heading 270
       
       ;; Define the agent shape according to its walking preferences
       ifelse pedestrians-profile = "Relaxed"     ;; "Relaxed" pedestrians do not want to reach their goal quickly
       [ 
         set shape "relaxed"
       ]
       [ ifelse pedestrians-profile = "Standard" ;; "Standard" pedestrians perform normal walking for different situations
          [ 
            set shape "peaton"
          ]
          [ if pedestrians-profile = "Hurried"  ;; "Relaxed" pedestrians do not want to reach their goal quickly
              [ 
                set shape "hurried"
              ]
          ]
       ]
       
       set speed (random 2)
       set x-goal -50
       set y-goal aux-y-goal
       set front-partners 0
       set lateral-partners 0
       set perceptual-field-i 0
       set front-perceptual-field "empty"
       set count-free-path 0
       set count-obs-path 0
       set count-pedestrian-enc 0
       set count-combined-enc 0
       set count-overlap 0
    ]
  ] 
  if(show-path?) [ ask turtles [ pen-down ] ]
end

to-report Pedestrians-walk:
   ;; Define the agent shape according to its walking preferences
       ifelse pedestrians-profile = "Relaxed"     ;; "Relaxed" pedestrians do not want to reach their goal quickly
       [ 
         report "Relaxed"
       ]
       [ ifelse pedestrians-profile = "Standard" ;; "Standard" pedestrians perform normal walking for different situations
          [ 
            report "Standard"
          ]
          [ if pedestrians-profile = "Hurried"  ;; "Relaxed" pedestrians do not want to reach their goal quickly
              [ 
                report "Hurried"
              ]
          ]
       ]
end
;;; Find an empty location to situate the pedestrian ;;;
to find-location [maxx maxy sign2]
  loop [
    set x-aux random (maxx) - (maxx / 2)
    set y-aux (random (maxy - 1) - (maxy / 2) + 1) 
    if count turtles with [xcor = x-aux and ycor = y-aux] = 0 [stop]
  ]
end

;;; Indicate preference factors of turn for different pedestrian walking rules ;;;
to preference-factors-turn
  ifelse pedestrians-profile = "Relaxed"     ;; "Relaxed" pedestrians do not want to reach their goal quickly
    [ 
      set pref-turn-free-path 20             ;; Performed many turns in a free path situation
      set pref-turn-obs-path 90              ;; Performed many turns in a obstructed path situation
      set pref-turn-pedes-encs 50            ;; Performed a few turns in a pedestrian encounter situation
      set pref-turn-combin-encs 50           ;; Performed a few turns in a combined encounter situation
    ]
    [ ifelse pedestrians-profile = "Standard" ;; "Standard" pedestrians perform normal walking for different situations
       [ 
         set pref-turn-free-path 6           
         set pref-turn-obs-path 50           
         set pref-turn-pedes-encs 70         
         set pref-turn-combin-encs 70        
       ]
       [ if pedestrians-profile = "Hurried"  ;; "Relaxed" pedestrians do not want to reach their goal quickly
           [ 
             set pref-turn-free-path 2       ;; Performed a few turns in a free path situation
             set pref-turn-obs-path 10       ;; Performed a few turns in a obstructed path situation
             set pref-turn-pedes-encs 90     ;; Performed many turns in a pedestrian encounter situation
             set pref-turn-combin-encs 90    ;; Performed many turns in a combined encounter situation
           ]
       ]
    ]
 end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;               P R O C E D U R E S     O F     T H E    A G E N T                     ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
turtles-own [ prev-x prev-y x-goal y-goal perceptual-field-i front-perceptual-field left-perceptual-field right-perceptual-field 
front-partners lateral-partners left-partner right-partner count-free-path count-obs-path count-pedestrian-enc count-combined-enc 
count-overlap dx-aux dy-aux occupied-front? occupied-left? occupied-right? rule sign step-size turn-size speed activity permitted-maneuvers ]

;---------------------------------------------------------------------
;   W A L K I N G    P R O C E S S    O F    P E D E S T R I A N S 
;---------------------------------------------------------------------
to walk
  ifelse (color = red and x-goal > xcor) or (color = blue and (x-goal - 1)< xcor)
    [ ; Perception module
      perceive
     
      ; Making decision module
      making-decision
       
      ; Actuation module
      perform-decision
    ] 
    [re-enter-corridor] ;; Periodic boundary conditions 
end

;----------------------------------------------------------------
;         P E R C E P T I O N     M O D U L E
;----------------------------------------------------------------
;;;  Each pedestrian examines constantly the state of its T-perceptual field,  ;;;
;;; starting from the nearer frontal square and finishing with lateral squares ;;;
to perceive
  front-perception
  lateral-perception
  if show-perceptual-field? [ perceptual-field-T ]
end

;;; Examine the frontal squares to find the nearest partner, wall or empty square ;;;
to front-perception
  let dx2 round(dx * 2)
  let dy2 round(dy * 2)
  let dx3 round(dx * 3)
  let dy3 round(dy * 3)
  let w-aux 0
  set perceptual-field-i 0
  set front-perceptual-field "empty"
  set occupied-front? false
  set count-overlap 0

  if count turtles-here  > 1
    [ set count-overlap 1 ]
    
  ;; Analize the first square of the T-perceptual field
  set front-partners count turtles-at round(dx) round(dy)
  set w-aux [ pcolor ] of patch-at round(dx) round(dy)      
  ifelse (front-partners > 0 and (who != one-of [who] of turtles-at round(dx) round(dy))) or w-aux = black 
    [ set occupied-front? true
      set perceptual-field-i 1
      ifelse w-aux = black 
        [ set front-perceptual-field "partner" ]
        [ set front-perceptual-field (round (one-of ([heading] of turtles-at round(dx) round(dy))))]
        set dx-aux round(dx)
        set dy-aux round(dy)
    ] 
    [ ;; If nothing is found within the first square, analize the second square
      set front-partners (count turtles-at dx2 dy2)
      set w-aux [pcolor] of patch-at dx2 dy2          
      ifelse (front-partners > 0 and (who != one-of [who] of turtles-at dx2 dy2)) or w-aux = black 
        [ set occupied-front? true
          set perceptual-field-i 2
          ifelse w-aux = black 
            [ set front-perceptual-field "partner" ]
            [ set front-perceptual-field (round (one-of ([heading] of turtles-at dx2 dy2))) ]
          set dx-aux dx2
          set dy-aux dy2
        ] 
        [ ;; If nothing is found within the second square, analize the third square
          set front-partners (count turtles-at dx3 dy3)
          set w-aux [pcolor] of patch-at dx3 dy3
          ifelse (front-partners > 0 and (who != one-of [who] of turtles-at dx3 dy3)) or w-aux = black 
          [ set occupied-front? true
            set perceptual-field-i 3
            ifelse w-aux = black 
              [ set front-perceptual-field "partner" ]
              [ set front-perceptual-field (round (one-of ([heading] of turtles-at dx3 dy3))) ]
            set dx-aux dx3
            set dy-aux dy3
          ] 
          [ set count-obs-path 0 set count-pedestrian-enc 0 set count-combined-enc 0 set count-free-path 0 ]
        ] 
    ]
end

;;; Examine both lateral squares to find a partner, wall or empty square ;;;
to lateral-perception
  let x-distance-left  round(sin (heading - 45)) 
  let y-distance-left  round(cos (heading - 45)) 
  let x-distance-right round(sin (heading + 45))
  let y-distance-right round(cos (heading + 45))
  
  let w-aux 0
  set left-perceptual-field "empty"
  set right-perceptual-field "empty"
  set occupied-left? false
  set occupied-right? false
  
  ;; Analize the left square of the T-perceptual field
  set left-partner count turtles-at x-distance-left y-distance-left
  set w-aux [ pcolor ] of patch-at x-distance-left y-distance-left    
  if (left-partner > 0 and (who != one-of [ who ] of turtles-at x-distance-left y-distance-left)) or w-aux = black [
    set occupied-left? true
    ifelse w-aux = black 
      [ set left-perceptual-field "partner"]
      [ set left-perceptual-field "v"
        set left-partner 1
      ]
  ] 
    
  ;; Analize the right square of the T-perceptual field
  set right-partner count turtles-at x-distance-right y-distance-right
  set w-aux [ pcolor ] of patch-at x-distance-right y-distance-right    
  if (right-partner > 0 and (who != one-of [who] of turtles-at x-distance-right y-distance-right)) or w-aux = black [
    set occupied-right? true     
    ifelse w-aux = black 
      [ set right-perceptual-field "partner"]
      [ set right-perceptual-field "v"
        set right-partner 1
      ]
  ] 
  set lateral-partners (left-partner + right-partner) 
end

;;; Show the T-perceptual field ;;;
to perceptual-field-T
  let dx1 dx 
  let dy1 dy 
  let dx2 dx * 2 
  let dy2 dy * 2 
  let dx3 dx * 3 
  let dy3 dy * 3 
  let patch-color-aux1   [ pcolor ] of patch-at dx dy
  let patch-color-aux2   [ pcolor ] of patch-at dx2 dy2
  let patch-color-aux3   [ pcolor ] of patch-at dx3 dy3
  let patch-color-left   [ pcolor ] of patch-at round(sin (heading - 45)) round(cos (heading - 45))
  let patch-color-right  [ pcolor ] of patch-at round(sin (heading + 45)) round(cos (heading + 45))

  ask patch-at dx1 dy1 [ set pcolor green]     
  ask patch-at dx2 dy2 [ set pcolor green ]  
  ask patch-at dx3 dy3 [ set pcolor green ]
  ask patch-at round (sin (heading - 45)) round (cos (heading - 45)) [ set pcolor 53 ]
  ask patch-at round (sin (heading + 45)) round (cos (heading + 45)) [ set pcolor 53 ]

     
  wait 0.1
  ask patch-at dx1 dy1 [ set pcolor patch-color-aux1 ]
  ask patch-at dx2 dy2 [ set pcolor patch-color-aux2 ]
  ask patch-at dx3 dy3 [ set pcolor patch-color-aux3 ]
  ask patch-at round (sin (heading - 45)) round (cos (heading - 45)) [ set pcolor patch-color-left ]
  ask patch-at round (sin (heading + 45)) round (cos (heading + 45)) [ set pcolor patch-color-right ]
end

;----------------------------------------------------------------
;      M A K I N G     D E C I S I O N     M O D U L E
;----------------------------------------------------------------
;;; Each pedestrian determines a new turn size and a new step size, applying the general walking rules ;;;
to making-decision
  ;; Walking sequence changes according to the orientation of the pedestrian
  ifelse aligned-to-borders? = 1
    [ decide-turn-forward-size ] 
    [ decide-forward-turn-size ]
end

;;; The pedestrian is aligned to the wall ;;;
to decide-turn-forward-size
  let total-partners (front-partners + left-partner + right-partner)
  set count-free-path 0
  
  ;; Different walking situations according to the occupation of the perceptual field of the pedestrian
  ifelse total-partners = 0  
    [ ;; The pedestrian does not perceive partners inside its T-perceptual field
      ifelse front-perceptual-field = "empty" and left-perceptual-field = "empty" and right-perceptual-field = "empty"
        [ free-path-aligned-borders 
          set count-free-path 1
        ]
        [ ifelse front-perceptual-field != "partner"
          [ obs-path-aligned-borders ]
          [ obs-path-non-aligned-borders ]
         set count-obs-path 1
        ]
    ]
    [;; The pedestrian perceives partners within its T-perceptual field
     ifelse front-perceptual-field != "partner" and left-perceptual-field != "partner" and right-perceptual-field != "partner"
       [ pedestrian-enc-aligned 
        set count-pedestrian-enc 1 
       ]
       [ if front-perceptual-field = "partner" or left-perceptual-field = "partner" or right-perceptual-field = "partner"
           [ifelse front-perceptual-field = "partner"
              [ non-aligned-comb-enc ]
              [ aligned-comb-enc ]           
           set count-combined-enc 1
          ]
       ]    
    ]
end

;;; The pedestrian is not aligned to the wall ;;;
to decide-forward-turn-size
  let total-partners (front-partners + left-partner + right-partner)
  set count-free-path 0
  
  ;; Different walking situations according to the occupation of the perceptual field of the pedestrian
  ifelse total-partners = 0
    [ ;; The pedestrian does not perceive partners within its T-perceptual field
      ifelse front-perceptual-field = "empty" and left-perceptual-field = "empty" and right-perceptual-field = "empty"
       [ free-path-non-aligned-borders 
         set count-free-path 1
       ]
       [ ifelse front-perceptual-field = "partner"
           [ obs-path-non-aligned-borders ]
           [ obs-path-aligned-borders ]
           set count-obs-path 1
       ]
    ]
    [ ;; The pedestrian perceives partners whitin its T-perceptual field
      ifelse front-perceptual-field != "partner" and left-perceptual-field != "partner" and right-perceptual-field != "partner"
        [ pedestrian-enc-non-aligned 
          set count-pedestrian-enc 1 
        ]
        [ if front-perceptual-field = "partner" or left-perceptual-field = "partner" or right-perceptual-field = "partner"
           [ ifelse front-perceptual-field = "partner" 
               [ non-aligned-comb-enc ]
               [ aligned-comb-enc ]  
            set count-combined-enc 1
           ]
        ]    
    ]
end

;----------------------------------------------------------------
;                W A L K I N G    R U L E S     
;----------------------------------------------------------------
;;; General walking rules to deal with a wide variety of perceived situations ;;;

;;; The value of turn size depends of the walking preference factor of the pedestrian.
;;; Thus, the value of turn size is selected randomly in any allowed direction according the 
;;; occupation of T-perceptual field. The different possible values of turn size are: +45, 0, -45
     
;;; The value of step size depends on the frontal occupancy of the squares of the pedestrian.
;;; Thus, if the pedestrian perceives any obstacle or partner in the first square, or if it perceives 
;;; any neighbor ahead in counter-flow, the value of step size is 0, in other case the value is 1

;------------ DIFFERENT SITUATIONS WHEN THE PEDESTRIAN IS ALIGNED TO THE CORRIDOR ------------;

;;; The pedestrian does not perceive any partner ;;;

;;; Free path rule, the pedestrian perceives nothing within its T-perceptual field ;;; 
to free-path-aligned-borders
  set rule (1 + random 100) 
  ;; Select the value of turn size randomly in one of 3 directions: +45, 0, -45 
  ifelse (rule >= 1 and rule <= pref-turn-free-path)
    [
      set turn-size 45 
      set step-size 1
    ]
    [ ifelse (rule > pref-turn-free-path and rule <= (100 - pref-turn-free-path))
        [
          set turn-size 0
          set step-size 1 + speed
        ]
        [
          set turn-size -45 
          set step-size 1
        ]
    ]
    set permitted-maneuvers 5 ;; The pedestrian has full maneuverability, i.e. 5 allowed maneuvers
end

;;; Obstructed path because of a wall perceived by the pedestrian  ;;;
to obs-path-aligned-borders 
  if (left-perceptual-field = "partner") or (right-perceptual-field = "partner") [
    ifelse (left-perceptual-field = "partner")
      [ set sign -1 ]
      [ set sign 1 ]
      
      set rule (1 + random 100)
      ;; Select the value of turn size randomly in one of the allowed directions
      ifelse (rule >= 0 and rule <= pref-turn-obs-path)
        [ 
          set turn-size (sign * 45) ; Select the value of turn size to get away from the wall
          set step-size 1
        ]
        [
          set turn-size 0           ; Select the value of turn size to remain oriented parallel to the wall
          set step-size 1
        ]
        set permitted-maneuvers 3   ; The pedestrian has restricted maneuverability, in this case 3 maneuvers
                                    ; are allowed because of an obstacle
    ]
end

;;; The pedestrian perceives some partners and it must apply a rule to solve such traffic problem using a set of collision avoidance rules ;;;
to pedestrian-enc-aligned
  ifelse front-partners > 0 
    [ ;; The pedestrian found a frontal partner
      let partner-flow? color = one-of [color] of turtles-at dx-aux dy-aux
      let enc-alineado false
   
      ifelse (((partner-flow? = false) and (angular-difference > 164 and angular-difference < 196)) or ((partner-flow? = true) and (angular-difference < 21)))
        [set enc-alineado true ]
        [set enc-alineado false]
   
      ifelse (left-partner + right-partner) = 0 
        [ ;; The pedestrian does not perceive any lateral partner
          front-non-laterals (enc-alineado) (partner-flow?)
        ]
        [ ifelse (left-partner + right-partner) = 1 
            [ ;; The pedestrian perceives one lateral partner
              front-one-lateral (partner-flow?) 
            ]
            [ ;; The pedestrian perceives two lateral partners
              front-two-laterals (partner-flow?)
            ] 
        ]          
    ]
    [ ;; The pedestrian does not perceive any frontal partner
      non-front-enc
    ]
end

;;; The pedestrian perceives different frontal or lateral partners ;;;
to front-non-laterals[ enc-alineado partner-flow? ]
  set rule (1 + random 100) 
  
  ;; Select the value of turn size randomly in any of the allowed directions
  ifelse enc-alineado = true
    [ ;; Aligned front encounter
      ifelse partner-flow? = true 
        [ ;; Flow encounter
          ifelse rule >= 1 and rule <= (pref-turn-pedes-encs / 2)
          [
            set turn-size 45
            set step-size 1
          ]
          [ ifelse rule > (pref-turn-pedes-encs / 2) and rule < (100 - (pref-turn-pedes-encs / 2))
              [
                set turn-size 0
                ifelse perceptual-field-i != 1 
                  [ set step-size 1 ]
                  [ set step-size 0 ]                 
              ]
              [
                set turn-size -45
                set step-size 1
              ]
          ] 
            ifelse perceptual-field-i != 1 
              [ set permitted-maneuvers 5 ]
              [ set permitted-maneuvers 4 ]
        ] 
        [ ;; Counter-flow encounter
          ifelse rule >= 1 and rule <= (pref-turn-pedes-encs / 2) ;en menos proporcion ???????????????????????????????????????????????????????????????
            [
              set turn-size 45
              set step-size 1
            ]
            [ ifelse rule > (pref-turn-pedes-encs / 2) and rule < (100 - (pref-turn-pedes-encs / 2))
                [ 
                  set turn-size 0
                  set step-size 0
                ]
                [
                  set turn-size -45
                  set step-size 1
                ]
            ]
              set permitted-maneuvers 4
        ]
    ] 
    [ ;; Non aligned frontal encounter
      let sign-pe -1 * (partner-sign)
      ifelse rule >= 1 and rule <= pref-turn-pedes-encs
        [
          set turn-size (sign-pe * 45)
          set step-size 1
        ]
        [
          ifelse partner-flow? = true and perceptual-field-i != 1
            [ 
              set turn-size 0
              set step-size 1 
              set permitted-maneuvers 3
            ]
            [ 
              set turn-size 0
              set step-size 0 
              set permitted-maneuvers 2
            ]
        ]         
    ]
end

to front-one-lateral[ partner-flow? ]
  let sign-pe pedestrian-sign-enc
  set rule (1 + random 100) 
  
  ;; Select the value of turn size randomly in any allowed direction
  ifelse rule >= 1 and rule <= pref-turn-pedes-encs
    [ 
      set turn-size (sign-pe * 45)
      set step-size 1
    ]
    [
      ifelse partner-flow? = true and perceptual-field-i != 1
        [ 
          set turn-size 0
          set step-size 1
          set permitted-maneuvers 3
        ]
        [
          set turn-size 0
          set step-size 0
          set permitted-maneuvers 2
        ]      
    ]
end

to front-two-laterals[partner-flow?]
  ;; The pedestrian can not turn due that its lateral squares are occupied
  set turn-size 0
  ifelse partner-flow? = true and perceptual-field-i != 1
    [ set step-size 1
      set permitted-maneuvers 1
    ]
    [ set step-size 0
      set permitted-maneuvers 0
    ]
end

to non-front-enc
  ifelse (left-partner + right-partner) = 1
    [ ;; The pedestrian perceives only one lateral partner
      let sign-pe pedestrian-sign-enc
      set rule (1 + random 100) 
      ifelse rule >= 1 and rule <= pref-turn-pedes-encs
        [ 
          set turn-size (sign-pe * 45)
          set step-size 1 
        ]
        [ 
          set turn-size 0
          set step-size 1
        ]        
        set permitted-maneuvers 3
      ]
      [ if (left-partner + right-partner) = 2 
          [ ;; The pedestrian perceives two lateral partners
            set turn-size 0
            set step-size 1
            set permitted-maneuvers 1
          ]
      ]
end

;;; The pedestrian perceives some partners and wall, thus it must avoid collisions ;;;
to aligned-comb-enc 
  ;; The pedestrian perceives wall within one of its lateral squares
  if (left-perceptual-field = "partner") or (right-perceptual-field = "partner")
    [ lateral-wall-lateral-or-front-partner ]
end

to lateral-wall-lateral-or-front-partner
  set sign pedestrian-sign-wall                 
  ifelse front-partners > 0
    [ ;;  The pedestrian perceives a frontl partner
      let partner-flow? color = one-of [color] of turtles-at dx-aux dy-aux
      
      ;; Select the value of turn size randomly in any allowed direction          
      ifelse (sign = -1) or (sign = 1) 
        [ ;; The opposite lateral square to the perceived wall is empty
          set rule (1 + random 100)                
          ifelse (rule >= 1 and rule <= pref-turn-combin-encs)
            [ 
              set turn-size (sign * 45) 
              set step-size 1
            ]
            [ 
              ifelse partner-flow? = true and perceptual-field-i != 1
                [
                  set turn-size 0
                  set step-size 1
                  set permitted-maneuvers 3
                ]
                [
                  set turn-size 0
                  set step-size 0
                  set permitted-maneuvers 2
                ]                  
            ]      
        ] 
        [ ;; The two lateral squares are occupied
          set turn-size 0
          ifelse partner-flow? = true and perceptual-field-i != 1
            [
              set step-size 1
              set permitted-maneuvers 1 
            ]
            [ 
              set step-size 0 
              set permitted-maneuvers 0
            ]
        ]
    ] 
    [ ;; Front squares are empty
      if sign = 0
        [
          set turn-size 0
          set step-size 1
          set permitted-maneuvers 1
        ]
    ]
end

;------------ DIFFERENT SITUATIONS WHEN THE PEDESTRIAN IS NOT ALIGNED TO THE CORRIDOR -----------------;
;;; Only the value of step size is determined because the pedestrian must be aligned to the corridor ;;;

;;; The pedestrian does not perceive any partner ;;;

;;; Free path rule, the pedestrian perceives that its T-perceptual field is empty ;;; 
to free-path-non-aligned-borders
  set step-size 1
  set permitted-maneuvers 2
end

;;; Obstructed path due the pedestrian perceives wall ;;;
to obs-path-non-aligned-borders
  if front-perceptual-field = "partner"
  [ ifelse perceptual-field-i != 1
      [ set step-size 1
        set permitted-maneuvers 2
      ]
      [ set step-size 0
        set permitted-maneuvers 1
      ]
  ]
end

;;; The pedestrian perceives some partners ;;;
to pedestrian-enc-non-aligned
  ifelse front-partners > 0
    [ let partner-flow? color = one-of [color] of turtles-at dx-aux dy-aux 
      ifelse partner-flow? = true and perceptual-field-i != 1
        [ set step-size 1
          set permitted-maneuvers 1
        ]
        [ set step-size 0
          set permitted-maneuvers 0
        ]
    ]
    [
      set step-size 1
      set permitted-maneuvers 1
    ]
    set permitted-maneuvers (permitted-maneuvers + 1)
end

;;; The pedestrian perceives some partners and wall ;;;
;;; Combined encounters ;;;
to non-aligned-comb-enc 
  ;; Forward permitted step
  ifelse perceptual-field-i != 1
    [ set step-size 1
      set permitted-maneuvers 2
    ]
    [ set step-size 0
      set permitted-maneuvers 1
    ]
end

;----------------------------------------------------------------
;            A C T U A T I O N     M O D U L E
;----------------------------------------------------------------
;;; Each pedestrian performs turn / forward to update its position in the corridor ;;;
to perform-decision     
  ifelse aligned-to-borders? = 1
    [ left turn-size   ;; Turn the decided angle
      fd step-size     ;; Go forward a permitted step
    ] 
    [ fd step-size     ;; Go forward a permitted step
      align-to-borders ;; Turn to align to the borders of the corridor
    ]
   
    ;; Go forward
    set xcor (round xcor)
    set ycor (round ycor)
    
    set step-size-sum (step-size-sum + step-size)
    calculate-activity
    set act-sum (act-sum + activity)
  
    ;; Some overlaps are unavoidable
    if count-overlap > 0 [
      let c-aux color
      set color black
      wait 0.1
      set color c-aux
    ]
end

;----------------------------------------------------------------
;      Periodic boundary conditions with different goal        
;----------------------------------------------------------------
to re-enter-corridor
  set y-goal (random ((corridor-width + 1) - 2)) - (((corridor-width + 1) - 2) / 2)
  set xcor (xcor * -1) 
end

;----------------------------------------------------------------
;        A U X I L I A R Y    F U N C T I O N S
;----------------------------------------------------------------
;;; Calculate the angular difference between the pedestrian and its perceived partner ;;;
to-report angular-difference
  report abs(round(front-perceptual-field - heading))
end

;;; Check the orientation with respect to the walls ;;;
to-report aligned-to-borders?
  ifelse (color = red and heading > 84 and heading < 96)
    [ report 1 ]
    [ ifelse (color = blue and heading > 264 and heading < 276)
        [ report 1 ]
        [ report 0 ]
    ] 
end

to align-to-borders
  ifelse color = red
    [ set heading 90 ]
    [ set heading 270 ]
end

to-report direction-walking?
  ifelse color = red
    [ report 90 ]
    [ report 270 ]
end

;;; Obtain the sign of the pedestrian according to its walking direction and the perceived situation ;;;
to-report partner-sign 
  ifelse (front-perceptual-field >= (direction-walking? - 45 - 6) ) and (front-perceptual-field <= (direction-walking? - 45 + 6) )
    [ report -1 ] 
    [ ifelse (front-perceptual-field >= (direction-walking? + 45 - 6) ) and (front-perceptual-field <= (direction-walking? + 45 + 6) )
        [ report 1 ] 
        [ report 0 ]
    ]
end

to-report pedestrian-sign-wall 
  ifelse left-perceptual-field = "partner"
    [ ifelse occupied-right? = false
        [ report -1 ]
        [ report 0 ]
    ] 
    [ if right-perceptual-field = "partner"
        [ ifelse occupied-left? = false
            [ report 1 ]
            [ report 0 ]
        ]
    ]
end

to-report pedestrian-sign-enc
  ifelse left-partner = 1
    [ report -1 ]
    [ if right-partner = 1 [ report 1 ] ]
end

;;; The activity represents the number of restrained maneuvers (of turn and step) ;;;
;;;       according the occupation of the perceptual field of each pedestrian        ;;;
to calculate-activity 
  ifelse permitted-maneuvers = 5
    [ set activity 0 ]
    [ ifelse permitted-maneuvers = 4
        [ set activity 0.2 ]
        [ ifelse permitted-maneuvers = 3
            [ set activity 0.4 ]
            [ ifelse permitted-maneuvers = 2
                [ set activity 0.6 ]
                [ ifelse permitted-maneuvers = 1
                    [ set activity 0.8 ]
                    [ if permitted-maneuvers = 0
                        [ set activity 1 ]
                    ]
                ]
            ]
        ]
    ]
end

; This model was created by:
; Ana Luisa Ballinas Hernndez, Jose Alejandro Rangel Huerta ( Benemrita Universidad Autnoma de Puebla )
; Anglica Muoz Melndez ( Intituto Nacional de Astrofsica ptica y Electrnica )
@#$#@#$#@
GRAPHICS-WINDOW
184
10
1204
251
50
10
10.0
1
10
1
1
1
0
1
1
1
-50
50
-10
10
0
0
1
Ticks

CC-WINDOW
5
537
1213
632
Command Center
0

PLOT
398
264
710
523
Average-speed-vs-density
Density
Average speed
0.0
10.0
0.0
1.0
true
false
PENS
"default" 1.0 0 -13345367 true

PLOT
828
263
1139
521
Average-activity-vs-density
Density
Average activity
0.0
1.0
0.0
1.0
true
false
PENS
"default" 1.0 0 -2674135 true

SLIDER
9
138
176
171
number-pedestrians
number-pedestrians
1
1900
10
1
1
NIL
HORIZONTAL

SLIDER
7
28
174
61
corridor-width
corridor-width
3
19
11
2
1
NIL
HORIZONTAL

BUTTON
52
82
130
115
setup
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL

BUTTON
51
322
128
355
go
ifelse(setup? = true and count turtles > 0 ) [go]\n [ show \"Error!\"\n   if(setup? != true)[show \"Necessary to press the setup button\" ]\n   if(count turtles = 0)[show \"Pedestrians have not been added to the corridor\" ]\n ]\n
T
1
T
OBSERVER
NIL
NIL
NIL
NIL

BUTTON
35
267
156
300
add-pedestrians
add-pedestrians
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL

SLIDER
15
482
183
515
Max-iterations
Max-iterations
100
500
200
1
1
NIL
HORIZONTAL

SWITCH
22
372
164
405
show-path?
show-path?
1
1
-1000

SWITCH
15
427
180
460
show-perceptual-field?
show-perceptual-field?
1
1
-1000

CHOOSER
24
197
162
242
pedestrians-profile
pedestrians-profile
"Relaxed" "Standard" "Hurried"
1

MONITOR
216
340
342
385
NIL
Pedestrians-walk:
17
1
11

@#$#@#$#@
WHAT IS IT?
-----------
This section could give a general understanding of what the model is trying to show or explain.


HOW IT WORKS
------------
This section could explain what rules the agents use to create the overall behavior of the model.


HOW TO USE IT
-------------
This section could explain how to use the model, including a description of each of the items in the interface tab.


THINGS TO NOTICE
----------------
This section could give some ideas of things for the user to notice while running the model.


THINGS TO TRY
-------------
This section could give some ideas of things for the user to try to do (move sliders, switches, etc.) with the model.


EXTENDING THE MODEL
-------------------
This section could give some ideas of things to add or change in the procedures tab to make the model more complicated, detailed, accurate, etc.


NETLOGO FEATURES
----------------
This section could point out any especially interesting or unusual features of NetLogo that the model makes use of, particularly in the Procedures tab.  It might also point out places where workarounds were needed because of missing features.


RELATED MODELS
--------------
This section could give the names of models in the NetLogo Models Library or elsewhere which are of related interest.


CREDITS AND REFERENCES
----------------------
This section could contain a reference to the model's URL on the web if it has one, as well as any other necessary credits or references.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

hurried
true
0
Polygon -7500403 true true 120 -15 0 285 300 285 180 -15 120 -15
Polygon -1 true false 150 90 90 105 150 15 210 105
Rectangle -1 true false 135 60 165 195

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

peaton
true
0
Circle -7500403 true true 15 13 272
Polygon -1 true false 150 90 90 105 150 15 210 105
Rectangle -1 true false 135 60 165 195

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

relaxed
true
0
Rectangle -7500403 true true 45 0 255 285
Polygon -1 true false 150 90 90 105 150 15 210 105
Rectangle -1 true false 135 60 165 195

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

t2
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 4.0.4
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
